@@ -18,11 +18,20 @@ along with CVXPY.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 import abc
-from ..atom import Atom
+from cvxpy.atoms.atom import Atom
 
 class Elementwise(Atom):
     """ Abstract base class for elementwise atoms. """
     __metaclass__ = abc.ABCMeta
     # The shape is the same as the argument's shape.
     def shape_from_args(self):
-        return self.args[0].shape
\ No newline at end of file
+        return self.args[0].shape
+
+    def validate_arguments(self):
+        """
+        Verify that all the shapes are the same
+        or can be promoted.
+        """
+        shape = self.args[0].shape
+        for arg in self.args[1:]:
+            shape = shape + arg.shape
\ No newline at end of file
