@@ -24,6 +24,7 @@ from cvxpy.expressions.variables import Variable
 from cvxpy.problems.objective import *
 from cvxpy.problems.problem import Problem
 import cvxpy.interface as intf
+import cvxpy.lin_ops.lin_utils as lu
 from base_test import BaseTest
 from cvxopt import matrix
 from numpy import linalg as LA
@@ -65,6 +66,37 @@ class TestProblem(BaseTest):
         params = p.parameters()
         self.assertItemsEqual(params, [p1, p2, p3])
 
+    def test_get_problem_data(self):
+        """Test get_problem_data method.
+        """
+        with self.assertRaises(Exception) as cm:
+            Problem(Maximize(exp(self.a))).get_problem_data(s.ECOS)
+        self.assertEqual(str(cm.exception), "Solver 'ECOS' cannot solve the problem.")
+
+        with self.assertRaises(Exception) as cm:
+            Problem(Maximize(exp(self.a))).get_problem_data(s.CVXOPT)
+        self.assertEqual(str(cm.exception), "Unsupported solver 'CVXOPT'.")
+
+        args = Problem(Maximize(exp(self.a) + 2)).get_problem_data(s.SCS)
+        data, dims = args
+        self.assertEqual(dims['ep'], 1)
+        self.assertEqual(data["c"].shape, (2,))
+        self.assertEqual(data["A"].shape, (3, 2))
+
+        args = Problem(Minimize(norm(self.x) + 3)).get_problem_data(s.ECOS)
+        c, G, h, dims, A, b = args
+        self.assertEqual(dims["q"], [3])
+        self.assertEqual(c.shape, (3,))
+        self.assertEqual(A.shape, (0, 3))
+        self.assertEqual(G.shape, (3, 3))
+
+        args = Problem(Minimize(norm(self.x) + 3)).get_problem_data(s.CVXOPT)
+        c, G, h, dims, A, b = args
+        self.assertEqual(dims["q"], [3])
+        self.assertEqual(c.size, (3, 1))
+        self.assertEqual(A.size, (0, 3))
+        self.assertEqual(G.size, (3, 3))
+
     # Test silencing and enabling solver messages.
     def test_verbose(self):
         # From http://stackoverflow.com/questions/5136611/capture-stdout-from-a-script-in-python
@@ -74,7 +106,7 @@ class TestProblem(BaseTest):
 
         # ####
         for verbose in [True, False]:
-            for solver in ["ecos", "cvxopt"]:
+            for solver in [s.ECOS, s.CVXOPT]:
                 sys.stdout = StringIO()     # capture output
                 p = Problem(Minimize(self.a), [self.a >= 2])
                 p.solve(verbose=verbose, solver=solver)
@@ -117,29 +149,37 @@ class TestProblem(BaseTest):
         num_solves = 4
         vars_lists = []
         ineqs_lists = []
+        var_ids_order_created = []
         for k in range(num_solves):
             sum = 0
             constraints = []
+            var_ids = []
             for i in range(100):
                 var = Variable(name=str(i))
+                var_ids.append(var.id)
                 sum += var
                 constraints.append(var >= i)
+            var_ids_order_created.append(var_ids)
             obj = Minimize(sum)
             p = Problem(obj, constraints)
-            objective, constr_map, dims = p.canonicalize()
-            all_ineq = itertools.chain(constr_map[s.EQ], constr_map[s.INEQ])
-            var_info = p._get_var_offsets(objective, all_ineq)
-            sorted_vars, var_offsets, x_length = var_info
-            vars_lists.append([int(v.name()) for v in sorted_vars])
-            ineqs_lists.append(constr_map[s.INEQ])
+            objective, constr_map, dims, solver = p.canonicalize(s.ECOS)
+            all_ineq = itertools.chain(constr_map[s.EQ], constr_map[s.LEQ])
+            var_offsets, var_sizes, x_length = p._get_var_offsets(objective, all_ineq)
+            # Sort by offset.
+            vars_ = sorted(var_offsets.items(), key=lambda (var_id, offset): offset)
+            vars_ = [var_id for (var_id, offset) in vars_]
+            vars_lists.append(vars_)
+            ineqs_lists.append(constr_map[s.LEQ])
 
         # Verify order of variables is consistent.
         for i in range(num_solves):
-            self.assertEqual(range(100), vars_lists[i])
+            self.assertEqual(var_ids_order_created[i],
+                vars_lists[i])
         for i in range(num_solves):
             for idx, constr in enumerate(ineqs_lists[i]):
-                var = constr.variables()[0]
-                self.assertEqual(idx, int(var.name()))
+                var_id, _ = lu.get_expr_vars(constr.expr)[0]
+                self.assertEqual(var_ids_order_created[i][idx],
+                    var_id)
 
     # Test removing duplicate constraint objects.
     def test_duplicate_constraints(self):
@@ -147,13 +187,20 @@ class TestProblem(BaseTest):
         le = (self.x <= 2)
         obj = 0
         def test(self):
-            objective,constr_map,dims = self.canonicalize()
-            return (len(constr_map[s.EQ]),len(constr_map[s.INEQ]))
+            objective,constr_map,dims,solver = self.canonicalize(s.ECOS)
+            return (len(constr_map[s.EQ]),len(constr_map[s.LEQ]))
         Problem.register_solve("test", test)
         p = Problem(Minimize(obj),[eq,eq,le,le])
         result = p.solve(method="test")
         self.assertEqual(result, (1,1))
 
+        # Internal constraints.
+        z = hstack(self.x, self.x)
+        obj = sum_entries(z[:,0] + z[:,1])
+        p = Problem(Minimize(obj))
+        result = p.solve(method="test")
+        self.assertEqual(result, (2,0))
+
     # Test the is_dcp method.
     def test_is_dcp(self):
         p = Problem(Minimize(normInf(self.a)))
@@ -325,6 +372,19 @@ class TestProblem(BaseTest):
         a.value = 2
         assert not (exp.value - 2*numpy.array([[1,2],[3,4]]).T).any()
 
+    def test_parameter_problems(self):
+        """Test problems with parameters.
+        """
+        p1 = Parameter()
+        p2 = Parameter(3, sign="negative")
+        p3 = Parameter(4, 4, sign="positive")
+        p = Problem(Maximize(p1*self.a), [self.a + p1 <= p2, self.b <= p3 + p3 + 2])
+        p1.value = 2
+        p2.value = -numpy.ones(3)
+        p3.value = numpy.ones((4, 4))
+        result = p.solve()
+        self.assertAlmostEqual(result, -6)
+
     # Test problems with normInf
     def test_normInf(self):
         # Constant argument.
@@ -411,9 +471,17 @@ class TestProblem(BaseTest):
         self.assertItemsAlmostEqual(self.x.value, [2,3])
         self.assertItemsAlmostEqual(self.z.value, [-1,-4])
 
+        # Row  arguments.
+        p = Problem(Minimize(norm2((self.x - self.z).T) + 5),
+            [self.x >= [2,3], self.z <= [-1,-4]])
+        result = p.solve()
+        self.assertAlmostEqual(result, 12.61577)
+        self.assertItemsAlmostEqual(self.x.value, [2,3])
+        self.assertItemsAlmostEqual(self.z.value, [-1,-4])
+
     # Test problems with abs
     def test_abs(self):
-        p = Problem(Minimize(sum(abs(self.A))), [-2 >= self.A])
+        p = Problem(Minimize(sum_entries(abs(self.A))), [-2 >= self.A])
         result = p.solve()
         self.assertAlmostEqual(result, 8)
         self.assertItemsAlmostEqual(self.A.value, [-2,-2,-2,-2])
@@ -481,7 +549,7 @@ class TestProblem(BaseTest):
         self.assertItemsAlmostEqual(p.constraints[1].dual_value, [-1, 0.5])
         self.assertAlmostEqual(p.constraints[2].dual_value, 0)
 
-        T = matrix(2,(2,3))
+        T = matrix(2, (2, 3))
         c = matrix([3,4])
         p = Problem(Minimize(1),
             [self.A >= T*self.C,
@@ -499,19 +567,18 @@ class TestProblem(BaseTest):
         p = Problem(Maximize(self.x[0,0]), [self.x[0,0] <= 2, self.x[1,0] == 3])
         result = p.solve()
         self.assertAlmostEqual(result, 2)
-        self.assertItemsAlmostEqual(self.x, [2,3])
+        self.assertItemsAlmostEqual(self.x.value, [2,3])
 
         n = 10
         A = matrix(range(n*n), (n,n))
         x = Variable(n,n)
-        p = Problem(Minimize(sum(x)), [x == A])
+        p = Problem(Minimize(sum_entries(x)), [x == A])
         result = p.solve()
         answer = n*n*(n*n+1)/2 - n*n
         self.assertAlmostEqual(result, answer)
 
         # Matrix variables
-        import __builtin__
-        p = Problem(Maximize( __builtin__.sum(self.A[i,i] + self.A[i,1-i] for i in range(2)) ),
+        p = Problem(Maximize( sum(self.A[i,i] + self.A[i,1-i] for i in range(2)) ),
                              [self.A <= [[1,-2],[-3,4]]])
         result = p.solve()
         self.assertAlmostEqual(result, 0)
@@ -526,18 +593,18 @@ class TestProblem(BaseTest):
 
     # Test problems with slicing.
     def test_slicing(self):
-        p = Problem(Maximize(sum(self.C)), [self.C[1:3,:] <= 2, self.C[0,:] == 1])
+        p = Problem(Maximize(sum_entries(self.C)), [self.C[1:3,:] <= 2, self.C[0,:] == 1])
         result = p.solve()
         self.assertAlmostEqual(result, 10)
         self.assertItemsAlmostEqual(self.C.value, 2*[1,2,2])
 
-        p = Problem(Maximize(sum(self.C[0:3:2,1])),
+        p = Problem(Maximize(sum_entries(self.C[0:3:2,1])),
             [self.C[1:3,:] <= 2, self.C[0,:] == 1])
         result = p.solve()
         self.assertAlmostEqual(result, 3)
         self.assertItemsAlmostEqual(self.C.value[0:3:2,1], [1,2])
 
-        p = Problem(Maximize(sum( (self.C[0:2,:] + self.A)[:,0:2] )),
+        p = Problem(Maximize(sum_entries( (self.C[0:2,:] + self.A)[:,0:2] )),
             [self.C[1:3,:] <= 2, self.C[0,:] == 1,
              (self.A + self.B)[:,0] == 3, (self.A + self.B)[:,1] == 2,
              self.B == 1])
@@ -565,7 +632,7 @@ class TestProblem(BaseTest):
         self.assertItemsAlmostEqual(self.A.value, [2,2,1,1])
 
         # Transpose of slice.
-        p = Problem(Maximize(sum(self.C)), [self.C[1:3,:].T <= 2, self.C[0,:].T == 1])
+        p = Problem(Maximize(sum_entries(self.C)), [self.C[1:3,:].T <= 2, self.C[0,:].T == 1])
         result = p.solve()
         self.assertAlmostEqual(result, 10)
         self.assertItemsAlmostEqual(self.C.value, 2*[1,2,2])
@@ -587,14 +654,14 @@ class TestProblem(BaseTest):
 
 
         c = matrix(1, (2,2))
-        p = Problem( Minimize( sum(vstack(self.A, self.C)) ),
+        p = Problem( Minimize( sum_entries(vstack(self.A, self.C)) ),
             [self.A >= 2*c,
             self.C == -2])
         result = p.solve()
         self.assertAlmostEqual(result, -4)
 
         c = matrix(1, (1,2))
-        p = Problem( Minimize( sum(vstack(c*self.A, c*self.B)) ),
+        p = Problem( Minimize( sum_entries(vstack(c*self.A, c*self.B)) ),
             [self.A >= 2,
             self.B == -2])
         result = p.solve()
@@ -604,22 +671,61 @@ class TestProblem(BaseTest):
         p = Problem( Minimize( c.T * vstack(square(self.a), sqrt(self.b))),
             [self.a == 2,
              self.b == 16])
+        with self.assertRaises(Exception) as cm:
+            p.solve()
+        self.assertEqual(str(cm.exception), "Problem does not follow DCP rules.")
+
+    # Test the hstack atom.
+    def test_hstack(self):
+        c = matrix(1, (1,5))
+        p = Problem(Minimize(c * hstack(self.x.T, self.y.T).T),
+            [self.x == [1,2],
+            self.y == [3,4,5]])
+        result = p.solve()
+        self.assertAlmostEqual(result, 15)
+
+        c = matrix(1, (1,4))
+        p = Problem(Minimize(c * hstack(self.x.T, self.x.T).T),
+            [self.x == [1,2]])
+        result = p.solve()
+        self.assertAlmostEqual(result, 6)
+
+
+        c = matrix(1, (2,2))
+        p = Problem( Minimize( sum_entries(hstack(self.A.T, self.C.T)) ),
+            [self.A >= 2*c,
+            self.C == -2])
+        result = p.solve()
+        self.assertAlmostEqual(result, -4)
+
+        c = matrix(1, (1,2))
+        p = Problem( Minimize( sum_entries(hstack(c*self.A, c*self.B)) ),
+            [self.A >= 2,
+            self.B == -2])
         result = p.solve()
         self.assertAlmostEqual(result, 0)
 
+        c = matrix([1,-1])
+        p = Problem( Minimize( c.T * hstack(square(self.a).T, sqrt(self.b).T).T),
+            [self.a == 2,
+             self.b == 16])
+        with self.assertRaises(Exception) as cm:
+            p.solve()
+        self.assertEqual(str(cm.exception), "Problem does not follow DCP rules.")
+
     # Test variable transpose.
     def test_transpose(self):
-        p = Problem(Minimize(sum(self.x)), [self.x.T >= matrix([1,2]).T])
+        p = Problem(Minimize(sum_entries(self.x)), [self.x.T >= matrix([1,2]).T])
         result = p.solve()
         self.assertAlmostEqual(result, 3)
         self.assertItemsAlmostEqual(self.x.value, [1,2])
 
-        p = Problem(Minimize(sum(self.C)), [matrix([1,1]).T*self.C.T >= matrix([0,1,2]).T])
+        p = Problem(Minimize(sum_entries(self.C)), [matrix([1,1]).T*self.C.T >= matrix([0,1,2]).T])
         result = p.solve()
         value = self.C.value
 
         constraints = [1*self.C[i,0] + 1*self.C[i,1] >= i for i in range(3)]
-        p = Problem(Minimize(sum(self.C)), constraints)
+        p = Problem(Minimize(sum_entries(self.C)), constraints)
         result2 = p.solve()
         self.assertAlmostEqual(result, result2)
         self.assertItemsAlmostEqual(self.C.value, value)
@@ -631,27 +737,27 @@ class TestProblem(BaseTest):
 
         exp = (-self.x).T
         print exp.size
-        p = Problem(Minimize(sum(self.x)), [(-self.x).T <= 1])
+        p = Problem(Minimize(sum_entries(self.x)), [(-self.x).T <= 1])
         result = p.solve()
         self.assertAlmostEqual(result, -2)
 
         c = matrix([1,-1])
-        p = Problem(Minimize(max(c.T, 2, 2 + c.T)[1]))
+        p = Problem(Minimize(max_elemwise(c.T, 2, 2 + c.T)[1]))
         result = p.solve()
         self.assertAlmostEqual(result, 2)
 
         c = matrix([[1,-1,2],[1,-1,2]])
-        p = Problem(Minimize(sum(max(c, 2, 2 + c).T[:,0])))
+        p = Problem(Minimize(sum_entries(max_elemwise(c, 2, 2 + c).T[:,0])))
         result = p.solve()
         self.assertAlmostEqual(result, 6)
 
         c = matrix([[1,-1,2],[1,-1,2]])
-        p = Problem(Minimize(sum(square(c.T).T[:,0])))
+        p = Problem(Minimize(sum_entries(square(c.T).T[:,0])))
         result = p.solve()
         self.assertAlmostEqual(result, 6)
 
         # Slice of transpose.
-        p = Problem(Maximize(sum(self.C)), [self.C.T[:,1:3] <= 2, self.C.T[:,0] == 1])
+        p = Problem(Maximize(sum_entries(self.C)), [self.C.T[:,1:3] <= 2, self.C.T[:,0] == 1])
         result = p.solve()
         self.assertAlmostEqual(result, 10)
         self.assertItemsAlmostEqual(self.C.value, 2*[1,2,2])
@@ -678,13 +784,13 @@ class TestProblem(BaseTest):
 
     # Test redundant constraints in cvxopt.
     def test_redundant_constraints(self):
-        obj = Minimize(sum(self.x))
+        obj = Minimize(sum_entries(self.x))
         constraints = [self.x == 2, self.x == 2, self.x.T == 2, self.x[0] == 2]
         p = Problem(obj, constraints)
         result = p.solve(solver=s.CVXOPT)
         self.assertAlmostEqual(result, 4)
 
-        obj = Minimize(sum(square(self.x)))
+        obj = Minimize(sum_entries(square(self.x)))
         constraints = [self.x == self.x]
         p = Problem(obj, constraints)
         result = p.solve(solver=s.CVXOPT)
@@ -724,9 +830,9 @@ class TestProblem(BaseTest):
     def test_expression_values(self):
         diff_exp = self.x - self.z
         inf_exp = normInf(diff_exp)
-        sum_exp = 5 + norm1(self.z) + norm1(self.x) + inf_exp
+        sum_entries_exp = 5 + norm1(self.z) + norm1(self.x) + inf_exp
         constr_exp = norm2(self.x + self.z)
-        obj = norm2(sum_exp)
+        obj = norm2(sum_entries_exp)
         p = Problem(Minimize(obj),
             [self.x >= [2,3], self.z <= [-1,-4], constr_exp <= 2])
         result = p.solve()
@@ -737,7 +843,7 @@ class TestProblem(BaseTest):
         self.assertItemsAlmostEqual(diff_exp.value, self.x.value - self.z.value)
         self.assertAlmostEqual(inf_exp.value,
             LA.norm(self.x.value - self.z.value, numpy.inf))
-        self.assertAlmostEqual(sum_exp.value,
+        self.assertAlmostEqual(sum_entries_exp.value,
             5 + LA.norm(self.z.value, 1) + LA.norm(self.x.value, 1) + \
             LA.norm(self.x.value - self.z.value, numpy.inf))
         self.assertAlmostEqual(constr_exp.value,
@@ -753,3 +859,12 @@ class TestProblem(BaseTest):
         p = Problem(obj)
         result = p.solve()
         self.assertAlmostEqual(result, 0)
+        assert self.a.value is not None
+
+    def test_div(self):
+        """Tests a problem with division.
+        """
+        obj = Minimize(normInf(self.A/5))
+        p = Problem(obj, [self.A >= 5])
+        result = p.solve()
+        self.assertAlmostEqual(result, 1)
